
%% MIMO GPC for Wood an Berry column
% Sergio Andres Castao Giraldo
% http://controlautomaticoeducacion.com/mimo-mpc/
% Universidade Federal de Rio de Janeiro
% Rio de Janeiro - 2017
% Universidade Federal de Santa Catarina
% Florianpolis - 2016
% Por Ecuaciones Diofantinas
%_____________________________________________________________________

clc
close all
clear all

%% Define o processo do reator:

Ps=[tf(12.8,[16.7 1]) tf(-18.9,[21 1]);tf(6.6,[10.9 1]) tf(-19.4,[14.4 1])];
%Ps=[minreal(tf(12.8,[16.7 1])/12.8) minreal(tf(-18.9,[21 1])/18.9);minreal(tf(6.6,[10.9 1])/6.6) minreal(tf(-19.4,[14.4 1])/19.4)];

%Ps.iodelay=[1 3;7 3];
Ps.iodelay=[0.5 1;1 0.5];

Ts=0.5;                     %Tiempo de Muestreo
Pz=c2d(Ps,Ts);              %Processo discreto

[Bp,Ap,dp]=descompMPC(Pz);     %Descompone num, den e atraso de Pz em celdas

%% Parametros de sintonia del GPC
%Encuentro cual es el retardo minimo en mi funcin de transferencia
dmin=[100 100]';
for i=1:2
    dmin(i)=min(dp(i,:));
end

N=[6;4];                              %Ventana de Prediccin
N1=[dmin(1)+1 dmin(2)+1];               %Horizonte Inicial
N2=[dmin(1)+N(1) dmin(2)+N(2)];         %Horizonte final
Nu=[2;2];                               %Horizonte de control
lambda=[10 100];                        %Ponderacin de la accin de control
delta=[1 1];                            %Ponderacin del seguimiento de referencia

%Matrices en bloque de los parametros de poderacin
Ql=blkdiag(lambda(1)*eye(Nu(1)),lambda(2)*eye(Nu(2)));  %(lambda -> Accin de Control)
Qd=blkdiag(delta(1)*eye(N(1)),delta(2)*eye(N(2)));      %(delta  -> Seguimiento de Referencia)

%% Calculo de la ecuacion Diofantina
%Se obtiene la matriz B y A (Numerador y denominador sistema MIMO)
% la matriz A es el minimo comn denominador del sistema
[B,A] = BA_MIMO(Bp,Ap);
%Funcin que Calcula Los Polinomios de la ecuacin Diofantina
[E,En,F] = diophantineMIMO(A,N,dmin);

%% Matriz de la Respuesta Forzada G
[G] = MatrizG(E,B,N,Nu,dp);

%Sistema sin Restricciones
M=inv(G'*Qd*G+Ql)*G'*Qd;    %Ganancia M
K1=M(1,:);                  %Tomo la primera columna unicamente (para control u1)
K2=M(Nu(1)+1,:);            %Tomo valores de M desde [N(1)+1] una muestra despues del primer 
                            %horizonte de control (para control u2)

%Determino el Vector con los controles pasados utilizando la funcion:
uG = deltaUFree(B,En,N,dp);

%% Loop de Controle
% inicializa parametros de Simulacion
nit=400;                        %Numero de interacciones
deltaU1(1:nit) = 0;             % Inicializa controle incremental
deltaU2(1:nit) = 0;

% Seales del processo MIMO 2x2 (Inicializacin)
%Salidas de cada bloque MIMO
yp11(1:nit) = 0; yp12(1:nit) = 0; yp21(1:nit) = 0; yp22(1:nit) = 0; 
%Salidas MIMO
y1(1:nit) = 0; y2(1:nit) = 0; 
%Referencias
r1(40:nit) = 0.8; r2(150:nit) = 0.4; 
%Acciones de Control
u1(1:nit) = 0; u2(1:nit) = 0; 

% Determino el tamao de los polinomios F
lduf1=size(F{1}); %Averiguo la longitud del polinomio F
lduf2=size(F{2}); %Averiguo la longitud del polinomio F
lduf1=lduf1(1,2);
lduf2=lduf2(1,2);
F1=F{1};
F2=F{2};

%_________ Vectores de Control Pasado _____________________________
%Determino el numero de columnas de los vectores pasados
luG11=size(uG{1,1}); 
luG12=size(uG{1,2}); 
luG21=size(uG{2,1}); 
luG22=size(uG{2,2}); 

luG11=luG11(1,2);
luG12=luG12(1,2);
luG21=luG21(1,2);
luG22=luG22(1,2);

duf11=zeros(1,luG11); %Vector incrementos de control pasados libre
duf12=zeros(1,luG12); %Vector incrementos de control pasados libre
duf21=zeros(1,luG21); %Vector incrementos de control pasados libre
duf22=zeros(1,luG22); %Vector incrementos de control pasados libre

%Comienzo el lazo de Control
for k=40:nit
     % Salida del proceso Real MIMO
     yp11(k) = transferencia2(Bp{1,1},Ap{1,1},dp(1,1),u1,yp11,k);
     yp12(k) = transferencia2(Bp{1,2},Ap{1,2},dp(1,2),u2,yp12,k);
     yp21(k) = transferencia2(Bp{2,1},Ap{2,1},dp(2,1),u1,yp21,k);
     yp22(k) = transferencia2(Bp{2,2},Ap{2,2},dp(2,2),u2,yp22,k);

     y1(k)=yp11(k)+yp12(k);
     y2(k)=yp22(k)+yp21(k);
     
%     % Tambien puedes simular el proceso de esta forma: 
%     t = 0:Ts:(k-1)*Ts;
%     yest=lsim(Ps,[u1(1:k);u2(1:k)],t,'zoh');
%     y1(k)=yest(end,1);
%     y2(k)=yest(end,2);
%         
     %Calculo de la respuesta libre
     free1=0;
     free2=0;
     for i=1:lduf1
         free1=free1+y1(k-i+1)*F1(:,i);
     end
     for i=1:lduf2
         free2=free2+y2(k-i+1)*F2(:,i);
     end
     
     %Adiciono vectores de controles pasados
     free1=free1+uG{1,1}*duf11'+uG{1,2}*duf12';
     free2=free2+uG{2,1}*duf21'+uG{2,2}*duf22';
        
     %Calculo del incremento de Control
     %Proyeto sin referencias futuras
     deltaU1(k)=K1*[(r1(k)*ones(1,length(free1(1,1:end)))'-free1);(r2(k)*ones(1,length(free2(1,1:end)))'-free2)];
     deltaU2(k)=K2*[(r1(k)*ones(1,length(free1(1,1:end)))'-free1);(r2(k)*ones(1,length(free2(1,1:end)))'-free2)];
     
     %Calculo de la ley de control
     if k==1
        u1(k)=deltaU1(k);
        u2(k)=deltaU2(k);
     else
        u1(k)=u1(k-1)+ deltaU1(k);
        u2(k)=u2(k-1)+ deltaU2(k);
     end
     
    %Actualizo vectores de Controles Pasados 
    aux_2=duf11(1:end-1);
    duf11=[deltaU1(k) aux_2];
    
    aux_2=duf12(1:end-1);
    duf12=[deltaU2(k) aux_2];
    
    aux_2=duf21(1:end-1);
    duf21=[deltaU1(k) aux_2];
    
    aux_2=duf22(1:end-1);
    duf22=[deltaU2(k) aux_2];

end

%Grafico Resultados
plotcontrol([y1;y2],[r1;r2],[u1;u2],Ts,nit);

% nm=nit;
% t = 0:Ts:(nm-1)*Ts;
% figure
% subplot(2,1,1),plot(t,r1,t,r2,t,y1,t,y2,'Linewidth',2)
% xlabel('Tempo (s)');
% ylabel('Saida');
% grid on;
% hold
% subplot(2,1,2),plot(t,u1,t,u2,'Linewidth',2)
% xlabel('Tempo (s)');
% ylabel('Controle');
% legend('u')
% grid on;
