
%% MIMO GPC for Wood an Berry column
% Sergio Andres Castao Giraldo
% http://controlautomaticoeducacion.com/mimo-mpc/
% Universidade Federal de Rio de Janeiro
% Rio de Janeiro - 2017
% Universidade Federal de Santa Catarina
% Florianpolis - 2016
% Por forma Recursiva
%_____________________________________________________________________

clc
close all
clear all

%% Define o processo do reator:

Ps=[tf(12.8,[16.7 1]) tf(-18.9,[21 1]);tf(6.6,[10.9 1]) tf(-19.4,[14.4 1])];

Ts=0.5;                     %Tiempo de Muestreo
Pz=c2d(Ps,Ts);              %Processo discreto

[Bp,Ap,dp]=descompMPC(Pz);     %Descompone num, den e atraso de Pz em celdas

%% Parametros de sintonia del GPC
%Encuentro cual es el retardo minimo en mi funcin de transferencia
dmin=[100 100]';
for i=1:2
    dmin(i)=min(dp(i,:));
end

N=[14;14];                              %Ventana de Prediccin
N1=[dmin(1)+1 dmin(2)+1];               %Horizonte Inicial
N2=[dmin(1)+N(1) dmin(2)+N(2)];         %Horizonte final
Nu=[12;12];                               %Horizonte de control
lambda=[1 1];                        %Ponderacin de la accin de control
delta=[1 1];                            %Ponderacin del seguimiento de referencia

%Matrices en bloque de los parametros de poderacin
Ql=blkdiag(lambda(1)*eye(Nu(1)),lambda(2)*eye(Nu(2)));  %(lambda -> Accin de Control)
Qd=blkdiag(delta(1)*eye(N(1)),delta(2)*eye(N(2)));      %(delta  -> Seguimiento de Referencia)


%% Calculo de la ecuacion Diofantina
%Se obtiene la matriz B y A (Numerador y denominador sistema MIMO)
% la matriz A es el minimo comn denominador del sistema
[Bt,A] = BA_MIMO(Bp,Ap);

%Calculo de las Diofantinas
[E,En,F] = diophantineMIMO(A,N,dmin);

%% Matriz de la Respuesta Forzada G
[G] = MatrizG(E,Bt,N,Nu,dp);

%Sistema sin Restricciones
M=inv(G'*Qd*G+Ql)*G'*Qd;    %Ganancia M
K1=M(1,:);                  %Tomo la primera columna unicamente (para control u1)
K2=M(Nu(1)+1,:);            %Tomo valores de M desde [N(1)+1] una muestra despues del primer 
                            %horizonte de control (para control u2)


%% Loop de Controle

%% inicializa parametros de Simulacion
nit=300; %Numero de interacciones

%Inicializa vector de las respuestas libres
y_free1=[];
y_free2=[];

%Inicializa vectores de los incrementos de control
deltaU1(1:nit) = 0;            
deltaU2(1:nit) = 0;

% Seales del processo MIMO 2x2 (Inicializacin)
%Salidas de cada bloque MIMO
yp11(1:nit) = 0; yp12(1:nit) = 0; yp21(1:nit) = 0; yp22(1:nit) = 0; 
yp(1:2)=0;
%Salidas MIMO
y1(1:nit) = 0; y2(1:nit) = 0; 
%Referencias
r1(60:nit) = 0.8; r2(150:nit) = 0.4; 
%Acciones de Control
u1(1:nit) = 0; u2(1:nit) = 0; 


%Atil (Producto entre polinomio A y el integrador)
At{1,1}=conv(A{1,1},[1 -1]);
At{2,2}=conv(A{2,2},[1 -1]);

%Determino el tamao de cada polinomio Atil
na1=length(At{1,1})-1;                       
na2=length(At{2,2})-1;   

%Determino el tamao de cada polinomio B proveniente del calculo del minimo
%comun denominador de la funcion "BA_MIMO"
nb1_11=length(Bt{1,1});                       
nb1_21=length(Bt{2,1});                       
nb1_12=length(Bt{1,2});                       
nb1_22=length(Bt{2,2});                       
%Comienzo el Lazo de Control
for k=40:nit
     % Salida del proceso Real
%         yp11(k) = transferencia2(Bp{1,1},Ap{1,1},dp(1,1),u1,yp11,k);
%         yp12(k) = transferencia2(Bp{1,2},Ap{1,2},dp(1,2),u2,yp12,k);
%         yp21(k) = transferencia2(Bp{2,1},Ap{2,1},dp(2,1),u1,yp21,k);
%         yp22(k) = transferencia2(Bp{2,2},Ap{2,2},dp(2,2),u2,yp22,k)
%         
%         y1(k)=yp11(k)+yp12(k);
%         y2(k)=yp22(k)+yp21(k);
%         
            t = 0:Ts:(k-1)*Ts;
            yest=lsim(Ps,[u1(1:k);u2(1:k)],t,'zoh');
            y1(k)=yest(end,1);
            y2(k)=yest(end,2);

      %% Creo los vectores que conforman la RESPUESTA LIBRE
      
      %Creo el vector de controles pasados basandome en la ecuacion de
      %salida del modelo de la planta actual y(k). O sea coloco en un vector
      %todos los incrementos de control de cada una de las entradas, algo como:
      %[deltaU(k-1) deltaU(k-2) ... deltaU(k-nb)] donde "nb" es
      %la longitud del vector B (Numerador del modelo)

      %Controles pasados de y1 en el instante actual
      deltaU11_pas=deltaU1(k-dp(1,1)-1:-1:k-dp(1,1)-nb1_11);           
      deltaU12_pas=deltaU2(k-dp(1,2)-1:-1:k-dp(1,2)-nb1_12); 
      %Controles pasados de y2 en el instante actual
      deltaU21_pas=deltaU1(k-dp(2,1)-1:-1:k-dp(2,1)-nb1_21);           
      deltaU22_pas=deltaU2(k-dp(2,2)-1:-1:k-dp(2,2)-nb1_22); 
      
      %Creo un vector de salidas pasadas, similar al vector de controles
      %pasados solo que en este caso con las salidas hasta la longitud del
      %polinomio A (denominador del modelo)
      y_pas1=y1(k:-1:k-na1+1);                         % Vetor de sada
      y_pas2=y2(k:-1:k-na2+1);                         % Vetor de sada
      
      %Ahora realizo mis predicciones hasta el horizonte de prediccin N2
      n=1; %Indice para la salida libre
      for j=1:N2(1)
          
          if j<=dp(1,1) 
              %Comienzo a desplazar los deltaU pasados a medida que aumento
              %la ventana de prediccin, hasta que la prediccin en este
              %vector llegue al retardo
              deltaU11_pas=[deltaU1(k-dp(1,1)-1+j) deltaU11_pas(1:end-1)];
          elseif j>dp(1,1) 
              %Una vez la ventana de prediccin es mayor que el retardo, en
              %la ecuacin de salida dejan de existir los deltaU pasados y
              %comenzaran a aparecer los deltaU futuros, por eso comienzo a
              %cerar los deltaU pasados.
              deltaU11_pas=[0 deltaU11_pas(1:end-1)];
          end
          if j<=dp(1,2) 
              deltaU12_pas=[deltaU2(k-dp(1,2)-1+j) deltaU12_pas(1:end-1)];
          elseif j>dp(1,2) 
              deltaU12_pas=[0 deltaU12_pas(1:end-1)];
          end
          
          %Hago el calculo de la respuesta libre con el polinomio Atil y el
          %polinomio B, junto con las salidas pasadas y los deltaU pasados
          y_free1(n)=-y_pas1*At{1,1}(2:end)'+deltaU11_pas*Bt{1,1}(1:end)'...
              +deltaU12_pas*Bt{1,2}(1:end)';% Armazena yfr
          
          %Actualizo el vector de salidas pasadas
          y_pas1=[y_free1(n) y_pas1(1:end-1)];                    % Atualiza yfr
          n=n+1;
      end
      

      %Ahora realizo mis predicciones hasta el horizonte de prediccin
      n=1; %Indice para la salida libre
      for j=1:N2(2)
          
          if j<=dp(2,1)
              deltaU21_pas=[deltaU1(k-dp(2,1)-1+j) deltaU21_pas(1:end-1)];
          elseif j>dp(2,1)
              deltaU21_pas=[0 deltaU21_pas(1:end-1)];
          end
          if j<=dp(2,2)
              deltaU22_pas=[deltaU2(k-dp(2,2)-1+j) deltaU22_pas(1:end-1)];
          elseif j>dp(2,2)
              deltaU22_pas=[0 deltaU22_pas(1:end-1)];
          end
          y_free2(n)=-y_pas2*At{2,2}(2:end)'+deltaU21_pas*Bt{2,1}(1:end)'...
              +deltaU22_pas*Bt{2,2}(1:end)';% Armazena yfr
          
          %Actualizo el vector de salidas pasadas
          y_pas2=[y_free2(n) y_pas2(1:end-1)];                    % Atualiza yfr
          n=n+1;
      end

     %% Calculo del Incremento de Control   
     %Calculo los incrementos de control utilizando unicamente los datos de
     %las respuestas libres que se encuentran dentro de los horizontes de
     %prediccin N1 y N2 y hago el producto con la ganancia calculada de la
     %respuesta forzada
    deltaU1(k)=K1*[(r1(k)*ones(1,length(y_free1(N1(1):N2(1))))'-y_free1(N1(1):N2(1))');(r2(k)*ones(1,length(y_free2(N1(2):N2(2))))'-y_free2(N1(2):N2(2))')];
    deltaU2(k)=K2*[(r1(k)*ones(1,length(y_free1(N1(1):N2(1))))'-y_free1(N1(1):N2(1))');(r2(k)*ones(1,length(y_free2(N1(2):N2(2))))'-y_free2(N1(2):N2(2))')];
     
    %% Calculo la accin de Control
     if k==1
        u1(k)=deltaU1(k);
        u2(k)=deltaU2(k);
     else
        u1(k)=u1(k-1)+ deltaU1(k);
        u2(k)=u2(k-1)+ deltaU2(k);
     end
     

end
% set_param('proceso','SimulationCommand','stop')
%% Grafico los Resultados
nm=nit;
t = 0:Ts:(nit-1)*Ts;
figure
subplot(2,1,1),plot(t,r1,t,r2,t,y1,t,y2,'Linewidth',2)
xlabel('Tempo (s)');
ylabel('Saida');
grid on;
hold
subplot(2,1,2),plot(t,u1,t,u2,'Linewidth',2)
xlabel('Tempo (s)');
ylabel('Controle');
legend('u')
grid on;
